#include "svg.h"
#include <iostream>
using namespace std;

void svg_begin(double width, double height) {
    cout << "<?xml version='1.0' encoding='UTF-8'?>\n";
    cout << "<svg width='" << width << "' height='" << height << "' "
         << "viewBox='0 0 " << width << " " << height << "' "
         << "xmlns='http://www.w3.org/2000/svg'>\n";

    //   SVG
    cout << "<style>\n"
         << "    .bar { fill: #4CAF50; stroke: #388E3C; stroke-width: 1; }\n"
         << "    .text { font: 12px sans-serif; fill: #333; }\n"
         << "</style>\n";
}

void svg_end() {
    cout << "</svg>\n";
}

void svg_text(double left, double baseline, string text) {
    cout << "<text class='text' x='" << left << "' y='" << baseline << "'>"
         << text << "</text>";
}

void svg_rect(double x, double y, double width, double height) {
    cout << "<rect class='bar' x='" << x << "' y='" << y << "' width='" << width
         << "' height='" << height << "' />";
}

void show_histogram_svg(const vector<size_t>& bins) {
    const auto IMAGE_WIDTH = 400;
    const auto IMAGE_HEIGHT = 300;
    const auto TEXT_LEFT = 20;
    const auto TEXT_BASELINE = 20;
    const auto TEXT_WIDTH = 50;
    const auto BIN_HEIGHT = 30;
    const auto MAX_WIDTH = IMAGE_WIDTH - TEXT_WIDTH;

    svg_begin(IMAGE_WIDTH, IMAGE_HEIGHT);

    double top = 0;
    size_t max_count = 0;
    for (size_t count : bins) {
        if (count > max_count) max_count = count;
    }

    for (size_t bin : bins) {
        const double bin_width = MAX_WIDTH * (static_cast<double>(bin) / max_count);
        svg_text(TEXT_LEFT, top + TEXT_BASELINE, to_string(bin));
        svg_rect(TEXT_WIDTH, top, bin_width, BIN_HEIGHT);
        top += BIN_HEIGHT;
    }

    svg_end();
}
